/*+-------------------------------------------------------------------+
  |                                                     iMAXalert.mq4 |
  |                                                                   |
  | "Modified Optimum Elliptic Filter"                                |
  | Source of calculations:                                           |
  | Stocks & Commodities vol 18:7 p20-29                              |
  | Optimal Detrending by John F. Ehlers                              |
  |                                                                   |
  |                                                  Crafted by Wylie |
  |                                              dazzle.html@live.com |
  +-------------------------------------------------------------------+*/

#property copyright "Wylie"
#property link      "dazzle.html@live.com"

#property indicator_chart_window
#property indicator_buffers 4

/*+-------------------------------------------------------------------+
  | iMAXalert Parameters                                              |
  +-------------------------------------------------------------------+*/

extern color    Phase1color   = Aqua;
extern color    Phase2color   = Red;
extern int      Ph2shift      = 1;
extern bool     EnableAlerts  = true;
extern bool     EnableArrows  = true;
extern int      ArrowUPsize   = 1;
extern int      ArrowDNsize   = 1;
extern color    ArrowUPcolor  = Aqua;
extern color    ArrowDNcolor  = Red;

double          ExtMapBuffer0[],ExtMapBuffer1[],CrossUp[],CrossDn[],Clearance;
int             MinBars,AST;
string          Chart;
static datetime AlertX[2];

/*+-------------------------------------------------------------------+
  | iMAXalert Initialization                                          |
  +-------------------------------------------------------------------+*/

int init()
{
 // Index Buffer 0

 SetIndexBuffer(0,ExtMapBuffer0);
 SetIndexShift(0,0);
 SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,1,Phase1color);

 // Index Buffer 1

 SetIndexBuffer(1,ExtMapBuffer1);
 SetIndexShift(1,0);
 SetIndexStyle(1,DRAW_LINE,STYLE_SOLID,1,Phase2color);

 // Index Buffer 2

 SetIndexBuffer(2,CrossUp);
 SetIndexStyle(2,DRAW_ARROW,STYLE_SOLID,ArrowUPsize,ArrowUPcolor);
 SetIndexArrow(2,225);

 // Index Buffer 3

 SetIndexBuffer(3,CrossDn);
 SetIndexStyle(3,DRAW_ARROW,STYLE_SOLID,ArrowDNsize,ArrowDNcolor);
 SetIndexArrow(3,226);
                                      // The "Clearance" variable below sets the distance from the trendline to the arrows...
                                      // It can be adjusted by the user to their own preference...
 switch(Period())
     {case PERIOD_M1:  Chart = "M1";  Clearance = 0.0003; AST =      59; break;
      case PERIOD_M5:  Chart = "M5";  Clearance = 0.0007; AST =     259; break;
      case PERIOD_M15: Chart = "M15"; Clearance = 0.0011; AST =     899; break;
      case PERIOD_M30: Chart = "M30"; Clearance = 0.0018; AST =    1799; break;
      case PERIOD_H1:  Chart = "H1";  Clearance = 0.0027; AST =    3599; break;
      case PERIOD_H4:  Chart = "H4";  Clearance = 0.0038; AST =   14399; break;
      case PERIOD_D1:  Chart = "D1";  Clearance = 0.0070; AST =   86399; break;
      case PERIOD_W1:  Chart = "W1";  Clearance = 0.0130; AST =  604799; break;
      case PERIOD_MN1: Chart = "MN1"; Clearance = 0.0260; AST = 2591999; break;}

 IndicatorShortName("iMAX");
 MinBars = 20;

 return (0);
} // int init()
  
/*+-------------------------------------------------------------------+
  | iMAXalert Main cycle                                              |
  +-------------------------------------------------------------------+*/

int start()
{    
 if(Bars <= MinBars){return(0);}

 double x = 0.5;
 int    countedBars = IndicatorCounted(),c,limit = Bars - countedBars - 1;

 if(countedBars < 0){return (-1);}
 if(countedBars > 0){countedBars--;}

 for(c = limit;c >= 0;c--)
     {ExtMapBuffer0[c] = (0.13785 * (2 * ((High[c]   + Low[c])   * x) - ((High[c+1] + Low[c+1]) * x)))
                       + (0.0007  * (2 * ((High[c+1] + Low[c+1]) * x) - ((High[c+2] + Low[c+2]) * x)))
                       + (0.13785 * (2 * ((High[c+2] + Low[c+2]) * x) - ((High[c+3] + Low[c+3]) * x)))
                       + (1.2103  * ExtMapBuffer0[c + 1] - 0.4867 * ExtMapBuffer0[c + 2]); 

      ExtMapBuffer1[c] = ExtMapBuffer0[c+Ph2shift];

      double b0 = ExtMapBuffer0[c],b1 = ExtMapBuffer0[c+1],
             r0 = ExtMapBuffer1[c],r1 = ExtMapBuffer1[c+1];

      if(EnableArrows)
         {if(b0 > r0 && b1 < r1)
             {CrossUp[c] = r0 - Clearance;}else{CrossUp[c] = EMPTY_VALUE;}
          if(b0 < r0 && b1 > r1)
             {CrossDn[c] = r0 + Clearance;}else{CrossDn[c] = EMPTY_VALUE;}}
     } // for(c = limit;c >= 0;c--)

 if(EnableAlerts)
    {if(b0 > r0 && b1 < r1)
        {sendAlert(0,(getDateTime()+" "+Symbol()+" "+Chart+" iMAX signals up trend crossing."));}
     if(b0 < r0 && b1 > r1)
        {sendAlert(1,(getDateTime()+" "+Symbol()+" "+Chart+" iMAX signals down trend crossing."));}}

 return (0);  
} // int start()

/*+-------------------------------------------------------------------+
  | End iMAXalert Main cycle                                          |
  +-------------------------------------------------------------------+*/

/*+-------------------------------------------------------------------+
  | iMAXalert support functions                                       |
  +-------------------------------------------------------------------+*/

/*+-------------------------------------------------------------------+
  | *** Alert processing function                                     |
  +-------------------------------------------------------------------+*/

void sendAlert(int AlertNum,string AlertText)
{
 if(TimeCurrent() > AlertX[AlertNum] + AST)
    {AlertX[AlertNum] = TimeCurrent();
                                        /* Add a sound file here if you want another form of alert...
                                           The sound file must be located in the MT4 "sounds" file folder 
                                           and must be a .wav format file. Also remove the // characters before
                                           the PlaySound instruction.  The Alert command and semicolon can 
                                           be removed if the user wishes to remove the alert entirely, if favor 
                                           of a sound file... or it can be "commented out" just add // before 
                                           the alert instruction. */

//     PlaySound("titleOFyourSoundfile.wav"); 
     Alert(AlertText);
    } // if(TimeCurrent() > AlertX[AlertNum] + AST)

 return(0);
} // void sendAlert(int AlertNum,string AlertText)

/*+-------------------------------------------------------------------+
  | *** Date/Time display function                                    |
  +-------------------------------------------------------------------+*/

string getDateTime()
{
 string dsplyDateTime = TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS);

 return(dsplyDateTime);
} // string getDateTime()

/*+-------------------------------------------------------------------+
  | End iMAXalert support functions                                   |
  +-------------------------------------------------------------------+*/


